from lxml.etree import parse, tostring, fromstring
from meresco.core import Observable
from urllib import urlopen
from urlparse import urlsplit
import re

from weightless import Suspend, identify
from socket import error as SocketError, SOL_SOCKET, SO_ERROR
import socket
from errno import EINPROGRESS
@identify
def doGet(host, port, request):
    this = yield # this generator, from @identify
    suspend = yield # suspend object, from Suspend.__call__
    res = socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)
    #print res[0]
    af, socktype, proto, canonname, sa = res[0]
    sok = socket.socket(af, socktype, proto)
    sok.setblocking(0)
    #sok.settimeout(1.0)
    try:
        sok.connect(sa)
    except SocketError, (errno, msg):
        if errno != EINPROGRESS:
            raise
    suspend._reactor.addWriter(sok, this.next)
    yield
    try:
        err = sok.getsockopt(SOL_SOCKET, SO_ERROR)
        if err != 0:    # connection created succesfully?
            raise IOError(err)
        yield
        suspend._reactor.removeWriter(sok)
        str  = 'GET %s HTTP/1.1\r\n' % request 
        str += 'Host: %s\r\n' % host
        str += '\r\n'
        #print str
        sok.sendall(str)
        suspend._reactor.addReader(sok, this.next)
        response = ""
        while True:
            yield
            line = sok.recv(4096) # error checking
            if line == '':
                break
            response += line
            # Transfer-Encoding: chunked -- wordt gebruikt door locator
            if response.endswith("0\r\n\r\n"): break
            # Gebruik anders Content-Length
        suspend._reactor.removeReader(sok)
        sok.close()
        suspend.resume(response)
    except Exception, e:
        suspend.throw(e)
    yield

def httpget(host, port, request):
    s = Suspend(doGet(host, port, request).send)
    yield s
    result = s.getResult()
    raise StopIteration(result)

class LoanInfo(Observable):
    def __init__(self, infra = None):
        Observable.__init__(self)
        self.infra = infra
        self.serverConfig  = self.infra.configRoot
        self.baseurl = self.serverConfig.get('baseurl', section='system')
        self.nsMap = self.serverConfig.getSectionDict('namespaces')
        self.availabilityConfig = self.infra.configRoot
        collectionString =  self.availabilityConfig.get('collectionList', section="collections")
        self.collections = []
        if collectionString: self.collections = collectionString.split("\n")
        exceptionString =  self.availabilityConfig.get('exceptionList', section="collections")
        self.exceptions = []
        if exceptionString: self.exceptions = exceptionString.split("\n")

    def okText(self):
       CRLF = "\r\n"
       ContentType = "Content-Type: text/html"
       okText = "HTTP/1.0 200 OK" + CRLF + \
                ContentType + CRLF + \
                CRLF
       return okText

    def handleRequest(self, RequestURI='', **kwargs):
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        ppn = Path.split("/loaninfo/")[1]
        yield self.okText()
        locations, genre = self.getEPNs(ppn)
        #print locations
        #print genre
        epn = ""
        for location in locations:
            if location["epn"]:
                epn = location["epn"]
                break
        http_response = yield httpget("search.uvt.nl", 80, "/locator1/Locator?epn=%(epn)s&sid=uvt-opc-uvt.nl&genre=%(genre)s" %{'epn': epn, 'genre': genre})
        body = http_response.split("\r\n\r\n")[1]
        # we gaan uit van een chunked response. 
        response = ""
        rest = body
        while True:
            s = rest.partition("\r\n")
            count = int(s[0], 16)
            chunk = s[2][0:count]
            response += chunk
            rest  = s[2][count+2:] # een chunk wordt afgesloten door CRLF
            if rest == "": break
        response = fromstring(response)
        #url = "http://search.uvt.nl/locator1/Locator?epn=%(epn)s&sid=uvt-opc-uvt.nl&genre=%(genre)s" %{'epn': epn, 'genre': genre}
        #response = parse(urlopen(url))
        #print tostring(response)
        yield """<html>
 <head>
  <title>LoanInfo for %s</title>
 <head>
 <body>
  <table width="100%%" border="1">
   <thead>
    <tr>
     <th>Location</th><th>Status</th><th>Shelfmark</th><th>Notes</th>
    </tr>
   </thead>
   <tbody>"""  %ppn
        for location in locations:
            if "electronicLocator" in location:
                status = self.availabilityConfig.get("onlineresource", section="wclSpecials", language='en')
                sgn    = ""
                msg    = '<img src="https://search.uvt.nl/files/edoc.gif" alt="*"/>&nbsp;<a href="%s">full text</a>'
            else: status, sgn, msg = self.getLoanInfo(location, response, genre)
            yield """
    <tr>
     <td>Library Tilburg University</td>
     <td>%(status)s</td>
     <td>%(sgn)s</td>
     <td>%(msg)s</td>
    </tr>""" %{'status': status, 'sgn': sgn, 'msg': msg}
        yield """   </tbody>
  </table>
  %s
 </body>
</html>""" %self.iframe(ppn)

    def getEPNs(self, ppn):
        recordId = "opc-uvt-nl:" + ppn
        mods = parse(self.any.getStream(recordId, 'mods'))
        #print tostring(mods)
        genre = mods.xpath("/mods:mods/mods:genre/text()", self.nsMap)[0]
        if genre.startswith("info:eu-repo/semantics/"):
            genre = genre.split("info:eu-repo/semantics/")[1]
        copies = mods.xpath("/mods:mods/mods:location/mods:holdingSimple/mods:copyInformation", self.nsMap)
        locations = []
        for copyInformation in copies:
            blok = {}
            EPN = copyInformation.xpath("mods:note[@type='epn']/text()", self.nsMap)[0]
            if EPN:
                blok["epn"] = EPN
                location = copyInformation.xpath("mods:shelfLocator/text()", self.nsMap)
                location = location and location[0] or ''
                if location:
                    blok["location"] = location
                    action = "stack"
                    if self.inDepot(location): action = "depot"
                    blok["action"] = action
                    try:
                        ulc = copyInformation.xpath("mods:note[@type='status']/text()", self.nsMap)[0]
                    except: ulc = ""
                    blok["ulc"] = ulc
                    notes = copyInformation.xpath("mods:note/text()", self.nsMap)
                    cdrom = ""
                    for note in notes:
                        expr = re.compile('CD[ -]ROM|DVD', re.IGNORECASE)
                        if expr.match(note): cdrom = "cdrom"
                    blok["cdrom"] = cdrom
                    if genre == "journal":
                        try:
                            holding = copyInformation.xpath("mods:enumerationAndChronology[@unitType='1']/text()", self.nsMap)[0]
                        except: holding = ""
                        blok["holding"] = holding
                    locations.append(blok)
                else:
                    electronicLocator = copyInformation.xpath("mods:electronicLocator/text()", self.nsMap)
                    electronicLocator = electronicLocator and electronicLocator[0] or ''
                    if electronicLocator:
                        blok["electronicLocator"] = electronicLocator
                        locations.append(blok)
        return locations, genre

    def getStatus(self, genre, availability, action, msg, ulc, cdrom):
        if ulc == "e": return self.availabilityConfig.get("onlineresource", section="wclSpecials", language='en'), msg
        availabilitymsg = msg
        if ulc   == "b": msg = self.availabilityConfig.get("kortuitleenbaar", section="availDisplayText", language='en')
        elif ulc == "f": msg = self.availabilityConfig.get("notforloan",      section="availDisplayText", language='en')
        if genre == "series": return self.availabilityConfig.get("seeitemsinseries", section="wclSpecials", language='en'), msg
        if genre == "journal":
            if action != "depot": return self.availabilityConfig.get("inlibrary", section="availabilityGroups", language='en'), msg
            if action == "depot": return "Request item from depot", msg
        if availability == "available" or availability == "possibly available" or availability == "unknown":
            if cdrom == "cdrom": return self.availabilityConfig.get("requestdesk", section="availabilityGroups", language='en'), msg
            if action != "depot":
                if availability == "possibly available": msg = "not for loan"
                return self.availabilityConfig.get("inlibrary", section="availabilityGroups", language='en'), msg
            if action == "depot":
                return self.availabilityConfig.get("indepot", section="availabilityGroups", language='en'), msg
        if availability == "not available":
           if "aangevraagd" in availabilitymsg or "Uitgeleend" in availabilitymsg:
               if ulc == "f": return self.availabilityConfig.get("requested", section="availabilityGroups", language='en'), msg
               else: return self.availabilityConfig.get("onloan", section="availabilityGroups", language='en'), msg
           if "Niet beschikbaar" in msg or "Niet uitleenbaar" in msg: return self.availabilityConfig.get("nietbeschikbaar", section="availabilityGroups", language='en'), msg
        return self.availabilityConfig.get("problem", section="availabilityGroups", language='en'), msg

    def inDepot(self, location):
        for exception in self.exceptions:
            if re.match(exception+'\s',location):
                return False
        for collection in self.collections:
            if re.match(collection+'\s',location):
                return True
        return False

    def iframe(self, ppn):
        if self.availabilityConfig.get("showrecord", section="wclSpecials") == "yes":
            s = '<br/><iframe src="%(baseurl)s/id/opc-uvt-nl:%(ppn)s" width="1000p" height="1000p"/>' %{'baseurl': self.availabilityConfig.get("baseurl", section="wclSpecials"), 'ppn': ppn}
        else: s = ""
        return s

    def getLoanInfo(self, location, response, genre):
        epn    = location["epn"]
        sgn    = location["location"]
        action = location["action"]
        cdrom  = location["cdrom"]
        ulc    = location["ulc"]
        nsMap = {'dlf': "http://diglib.org/ilsdi/1.1"}
        path = "/records/record[epn='%s']/availability/dlf:collection/dlf:record/dlf:simpleavailability" %epn
        try:
            avail = response.xpath(path, nsMap)[0]
        except: pass
        try:
            availability = avail.xpath("dlf:availabilitystatus/text()", nsMap)[0]
        except: availability = ""
        #print availability
        try:
            msg = avail.xpath("dlf:availabilitymsg/text()", nsMap)[0]
        except: msg = ""
        if genre == "journal":
            path = "/records/record[epn='%s']/action/text()" %location["epn"]
            action = response.xpath(path, nsMap)[0]
        status, msg = self.getStatus(genre, availability, action, msg, ulc, cdrom)
        if genre == "journal":
            path = "/records/record[epn='%s']/collection/text()" %location["epn"]
            collection = response.xpath(path, nsMap)[0]
            msg = location["holding"] + " (" + collection + ") - " + msg
        return status, sgn, msg
